@@ -36,9 +36,12 @@ static struct option longopts[] = {
     { "radio",       0, 0, 'r' },
     { "control",     1, 0, 'c' },
     { "display",     1, 0, 'd' },
+    { "reinit",      1, 0, 'D' },
     { "temp",        0, 0, 't' },
+    { "timetemp",    0, 0, 'T' },
     { "update-temp", 0, 0, 'u' },
     { "volume",      0, 0, 'l' },
+    { "vol",         0, 0, 'L' },
     { "manual",      0, 0, 'm' },
     { "time",        0, 0, 's' },
     { "version",     0, 0, 'v' },
@@ -82,9 +85,13 @@ static void help(char *argv0)
 "  (2) -d \"123456 fm pause both\" will display 12 34:56 and also light up the fm and pause leds.\n"
 "  N.B. You cannot have multiple mode LEDs lit at the same time (e.g. am + fm, or play + pause).\n"
 "\n"
+"  -D, --reinit [\"xxxxxx\"]  As above, but re-initilises the display first\n"
+"\n"
 "  -t, --temp                 Display the current temperature.\n"
+"  -T, --timetemp             Display both the current temperature and time.\n"
 "  -u, --update-temp          Updates the temperature every 2 seconds.\n"
 "  -l, --volume               Displays any changes on the volume control.\n"
+"  -L, --vol                  As above, but displays no other text.\n"
 "  -m, --manual               Allows you to interactively change a register.\n"
 "  -s, --time                 Sets and displays the current local time.\n"
 "\n"
@@ -203,10 +210,11 @@ void digimatrix_TurnOnRadio(int file, RADIO_MODE radio_mode)
     }
 }
 
-void digimatrix_DisplayNumbers(int file, char digits[6], LED led, PP pp, DOTS dots)
+void digimatrix_DisplayNumbers(int file, char digits[6], LED led, PP pp, DOTS dots, BOOL reinit)
 {
     int n;
     
+    if (reinit)
     digimatrix_write_byte_data(file, 0x23, 0x02); // Set to digit mode (-- --:--).
     
     // Set up the Dots.
@@ -269,6 +277,13 @@ void digimatrix_DisplayNumbers(int file, char digits[6], LED led, PP pp, DOTS do
                 case 'M':
                     digimatrix_write_byte_data(file, n, 0x0c);
                     break;
+                
+		case '-':
+		    if (n == 1)
+                    	digimatrix_write_byte_data(file, n, 0x0d);
+		    else
+                    	digimatrix_write_byte_data(file, n, 0x0c);
+                    break;
 
                 default:
                     digimatrix_write_byte_data(file, n, 0x0f);
@@ -309,8 +324,12 @@ void digimatrix_DisplayTemp(int file, int temp)
     //digimatrix_write_byte_data(file, 0x0f, 0x01);
 }
 
-void digimatrix_UpdateTemp(int file, int temp)
+/* Note: Returns hexified version of temperature: 
+         Temp = 34 DegC, return will be 0x34 */
+
+char digimatrix_GetTemp()
 {
+    int temp;
     char temp_dec = 0;
     // Get the temperature from the IT87
 
@@ -325,11 +344,17 @@ void digimatrix_UpdateTemp(int file, int temp)
         temp = inb(0x296);
     }
 
-                    
     temp &= 0xFF;
     temp_dec = ((temp / 10) << 4) & 0xf0; 
     temp_dec |= (temp % 10) & 0x0f;
-    digimatrix_write_byte_data(file, 0x22, temp_dec);
+
+    return (temp_dec);
+}
+
+void digimatrix_UpdateTemp(int file, int temp)
+{
+    printf("Temperature = %d\n\r", digimatrix_GetTemp());
+    digimatrix_write_byte_data(file, 0x22, digimatrix_GetTemp());
 }
 
 static int digimatrix_I2CInit(int i2cbus)
@@ -423,19 +448,19 @@ static int digimatrix_I2CInit(int i2cbus)
 
 int main(int argc, char *argv[])
 {
-  char digits[6];
+  char digits[7];
   int file ; //= digimatrix_I2CInit(0);;
   int ch, longindex = 0;
 
   char buffer[20], *ptr;
 
   BOOL bUpdateTemp = FALSE;
-  BOOL bUpdateVol = FALSE;
+  BOOL bUpdateTime = FALSE;
 
   const struct tm * nice_time;
   time_t nasty_time;
   char hour, minute;
-
+  int volume;
 
   if (argc == 1)
   {
@@ -452,21 +477,20 @@ int main(int argc, char *argv[])
 */
     for (;;) 
     {
-        if ((ch = getopt_long(argc, argv, "i:r:d:c:tulmsvh", longopts, &longindex)) == -1) 
+        if ((ch = getopt_long(argc, argv, "i:r:d:D:c:tTulLmsvh", longopts, &longindex)) == -1) 
         {
             break;
         }
 
         if ( (bI2CInitialised != TRUE) && (ch!='i' && ch!='h' && ch!='v') )
         {
-            printf("\n*** You must supply -i <n> as the first arguement! ***\n\n");
-            break;
+	    printf("\n*** You must supply -i <n> as the first arguement! ***\n\n");
         }
 
         switch (ch) 
         {
         case 'i':
-            printf("\nUsing I2C Bus %s\n", optarg);
+            //printf("\nUsing I2C Bus %s\n", optarg);
             file = digimatrix_I2CInit(atoi(optarg));
             if (file > 0)
             {
@@ -535,6 +559,7 @@ int main(int argc, char *argv[])
  
             
         case 'd':
+        case 'D':
 	    {
 	        LED led = LED_NONE;
 		PP pp = PP_NONE;
@@ -544,33 +569,33 @@ int main(int argc, char *argv[])
                 
 		printf("Args = %s", optarg);
                 printf("\nSetting display to: %s ", digits);
-    	        if (strstr(optarg, "play"))
+    	        if (strstr(optarg, " play"))
    	        {
 		    pp = PP_PLAY;
 		    printf(" play");
                 }
-    	        else if (strstr(optarg, "pause"))
+    	        else if (strstr(optarg, " pause"))
    	        {
 		    pp = PP_PAUSE;
 		    printf(" pause");
                 }
 		
-    	        if (strstr(optarg, "am"))
+    	        if (strstr(optarg, " am"))
    	        {
 		    led = LED_AM;
 		    printf(" am");
                 }
-    	        else if (strstr(optarg, "fm"))
+    	        else if (strstr(optarg, " fm"))
    	        {
 		    led = LED_FM;
 		    printf(" fm");
                 }
-    	        else if (strstr(optarg, "cd"))
+    	        else if (strstr(optarg, " cd"))
    	        {
 		    led = LED_CD;
 		    printf(" cd");
                 }
-    	        else if (strstr(optarg, "hd"))
+    	        else if (strstr(optarg, " hd"))
    	        {
 		    led = LED_HD;
 		    printf(" hd");
@@ -593,14 +618,21 @@ int main(int argc, char *argv[])
 		    printf(" .");
                 }
 		printf("\n");
-		
-                digimatrix_DisplayNumbers(file, digits, led, pp, dots);
+		if (ch == 'D')
+                digimatrix_DisplayNumbers(file, digits, led, pp, dots, 1);
+		else
+                digimatrix_DisplayNumbers(file, digits, led, pp, dots, 0);
 	    }
             break;
             
         case 't':
             printf("Displaying Temperature.\n");
             digimatrix_DisplayTemp(file, 99);
+	    break;
+	    
+        case 'T':
+	    bUpdateTime = TRUE;
+	    bUpdateTemp = TRUE;
             break;
             
         case 'u':
@@ -610,8 +642,11 @@ int main(int argc, char *argv[])
             break;
             
         case 'l':
-            printf("Display Volume control. \n");
-            bUpdateVol = TRUE;
+            printf("display volume control. \nVolume Change: ");
+	case 'L':
+            digimatrix_write_byte(file, 0x24);
+            volume = digimatrix_read_byte(file);
+            printf("%d\n", volume);
             break;
 
         case 'm':
@@ -667,15 +702,40 @@ int main(int argc, char *argv[])
         }
     }
 
-    if (bUpdateVol || bUpdateTemp)
+    if (bUpdateTemp)
     {
         unsigned int temp_counter = 0;
-        int res, old_res = 0xDEADF00D;
-	char volume = 0;
 
         while (1)
         {
-            if (bUpdateTemp)
+            if (bUpdateTemp && bUpdateTime)
+            {
+                if (temp_counter >= (TEMPERATURE_UPDATE_INTERVAL_MS / VOLUME_UPDATE_INTERVAL_MS))
+                {
+		    time_t current_time;
+		    struct tm *time_date;
+	            LED led = LED_NONE;
+		    PP pp = PP_NONE;
+		    static DOTS dots = DOTS_NONE;
+		    char ascii_time[5];
+
+                    // Make the dots flash. 
+                    if (dots == DOTS_NONE)
+		        dots = DOTS_BOTH;
+		    else
+		        dots = DOTS_NONE;
+			
+                    temp_counter = 0;
+		    
+		    time(&current_time);
+		    time_date = localtime(&current_time);
+		    strftime(ascii_time, 5, "%H%M", time_date);
+		    
+		    sprintf(digits, "%02X%s", digimatrix_GetTemp(), ascii_time);
+                    digimatrix_DisplayNumbers(file, digits, led, pp, dots, 0);
+                }
+            }
+            else if (bUpdateTemp)
             {
                 if (temp_counter >= (TEMPERATURE_UPDATE_INTERVAL_MS / VOLUME_UPDATE_INTERVAL_MS))
                 {
@@ -685,17 +745,6 @@ int main(int argc, char *argv[])
                 }
             }
 
-            if (bUpdateVol)
-            {
-                digimatrix_write_byte(file, 0x24);
-                //fprintf(stderr, "%d - %d            \r", digimatrix_read_byte(file), n);
-                res = volume = digimatrix_read_byte(file);
-                if (res != 0) //old_res != res)
-                {
-                    old_res = res;
-                    fprintf(stderr, "Volume Change: %d\n", res);
-                }
-            }
             usleep(VOLUME_UPDATE_INTERVAL_MS * 1000); // Delay of 500ms
             temp_counter++;
         }
